(function ($) {
  Drupal.behaviors.pwa_firebase = {
    attach: function (context, settings) {
      // [START get_messaging_object]
      // Retrieve Firebase Messaging object.
      if (!settings.pwa_firebase.firebaseConfig) {
        return;
      }
      firebase.initializeApp(settings.pwa_firebase.firebaseConfig);
      if (firebase.analytics != undefined) {
        firebase.analytics();
      }
      messaging = firebase.messaging();
      // [END get_messaging_object]
      // [START set_public_vapid_key]
      // Add the public key generated from the console here.
      messaging.usePublicVapidKey(settings.pwa_firebase.VapidKey);
      // [END set_public_vapid_key]
      // [START refresh_token]
      // Callback fired if Instance ID token is updated.
      messaging.onTokenRefresh(() => {
        messaging.getToken().then((refreshedToken) => {
          //console.log('Token refreshed.');
          // Indicate that the new Instance ID token has not yet been sent to
          // the app server.
          setTokenSentToServer(false);
          // Send Instance ID token to app server.
          sendTokenToServer(refreshedToken);
          // [START_EXCLUDE]
          // Display new Instance ID token and clear UI of all previous
          // messages.
          resetUI();
          // [END_EXCLUDE]
        }).catch((err) => {
          console.log('Unable to retrieve refreshed token ', err);
        });
      });
      // [END refresh_token]

      // [START receive_message]
      // Handle incoming messages. Called when:
      // - a message is received while the app has focus
      // - the user clicks on an app notification created by a service worker
      //   `messaging.setBackgroundMessageHandler` handler.
      messaging.onMessage((payload) => {
        //console.log('Message received. ', payload);
        // [START_EXCLUDE]
        // Update the UI to include the received message.
        appendMessage(payload);
        // [END_EXCLUDE]
        // [END receive_message]
      });
      resetUI();

      // IDs of divs that display Instance ID token UI or request permission UI.
      const permissionDivId = 'pwa_firebase_notifications';
      const tokenDivId = 'pwa_firebase_remove_token';

      $("#" + permissionDivId).on("click", function () {
        requestPermission();
      });
      $("#" + tokenDivId).on("click", function () {
        deleteToken();
      });

      function showPopup() {
        let dialog = '<div class="text-center dialog">' + Drupal.t("This application will send notification to your device") +
          '<div class=" ml-2 d-flex align-items-center justify-content-center g-2">' +
            '<button class="allow-button btn btn-success mr-1" id="' + permissionDivId + '">' + Drupal.t('Allow') + '</button> ' +
          '</div></div>';
        let notify = $.notify(dialog, {allow_dismiss: true});
      }
      let isShowPopup = sessionStorage.getItem('pwa_firebase_popup');
      if(!isShowPopup){
        showPopup();
        sessionStorage.setItem('pwa_firebase_popup', true);
      }

      function resetUI() {
        clearMessages();
        // Get Instance ID token. Initially this makes a network call, once
        // retrieved subsequent calls to getToken will return from cache.
        messaging.getToken().then((currentToken) => {
          console.log('current Token',currentToken);
          if (currentToken) {
            sendTokenToServer(currentToken);
            updateUIForPushEnabled(currentToken);
          } else {
            // Show permission request.
            console.log('No Instance ID token available. Request permission to generate one.');
            // Show permission UI.
            updateUIForPushPermissionRequired();
            setTokenSentToServer(false);
          }
        }).catch((err) => {
          console.log('An error occurred while retrieving token. ', err);
          setTokenSentToServer(false);
        });
      }

      // Send the Instance ID token your application server, so that it can:
      // - send messages back to this app
      // - subscribe/unsubscribe the token from topics
      function sendTokenToServer(currentToken) {
        if (!isTokenSentToServer()) {
          $.post("/firebase-send-token", {token: currentToken}, function (data) {
            //console.info('token send to server', data);
          });
          setTokenSentToServer(true);
        } else {
          //console.log('Token already sent to server so won\'t send it again
          // unless it changes');
        }
      }

      function isTokenSentToServer() {
        return window.localStorage.getItem('sentToServer') === '1';
      }

      function setTokenSentToServer(sent) {
        window.localStorage.setItem('sentToServer', sent ? '1' : '0');
      }

      function requestPermission() {
        console.log('Requesting permission...');
        Notification.requestPermission().then((permission) => {
          if (permission === 'granted') {
            //If notification is allowed
            navigator.serviceWorker.ready.then(p => {
              p.pushManager.getSubscription().then(subscription => {
                if (subscription === null) {
                  //If there is no notification subscription, register.
                  let re = p.pushManager.subscribe({userVisibleOnly: true})
                }
              })
            });
            resetUI();
          } else {
            //If notification is not allowed
            console.log('Unable to get permission to notify.' + permission);
          }
        });
      }

      function deleteToken() {
        // Delete Instance ID token.
        // [START delete_token]
        messaging.getToken().then((currentToken) => {
          messaging.deleteToken(currentToken).then(() => {
            //console.log('Token deleted.');
            setTokenSentToServer(false);
            // [START_EXCLUDE]
            // Once token is deleted update UI.
            resetUI();
            // [END_EXCLUDE]
          }).catch((err) => {
            console.log('Unable to delete token. ', err);
          });
          // [END delete_token]
        }).catch((err) => {
          console.log('Error retrieving Instance ID token. ', err);
        });
      }

      // Add a message to the messages element.
      function appendMessage(payload) {
        let notification = payload.notification;
        let badge = payload.data.badge;
        let title = notification.title;
        let body = notification.body;
        let url = payload.data.link;
        let bg = payload.data.bg;
        $.notify({
          // options
          icon: 'glyphicon glyphicon-warning-sign',
          target: '_blank',
          url: url,
          title: title,
          message: body
        }, {
          // settings
          type: bg
        });

        let selectorMessage = settings.pwa_firebase.idmessage;
        if (url) {
          title = '<a href="' + url + '">' + title + '</a>';
          body = '<a href="' + url + '">' + body + '</a>';
        }
        if (selectorMessage) {
          if (!$(selectorMessage + ' .dropdown-menu').length) {
            $(selectorMessage).addClass("dropdown").append('<span class="badge badge-danger">' + badge + '</span><ul class="notification dropdown-menu"></ul>');
          }
          $(selectorMessage).find('ul').append('<li>' + title + '<hr/>' + body + '</li>');
        }
        let idRandom = Math.ceil(Math.random() * 100);
        $('#pwa_firebase_message').append('<div class="panel panel-default" id="messages-' + idRandom + '">' +
          '  <div class="panel-heading">' + title +
          '<button type="button" class="close" data-target="#messages-' + idRandom + '" data-dismiss="alert"> ' +
          ' <span aria-hidden="true">×</span><span class="sr-only">Close</span>\n' +
          '</button></div><div class="panel-body">' + body + '</div></div>');
        //todo store all message in storage browser
      }

      // Clear the messages element of all children.
      function clearMessages() {
        let selectorMessage = settings.pwa_firebase.idmessage;
        $(selectorMessage + ' .badge').html('');
        $(selectorMessage + ' .notification').html('');
      }

      function updateUIForPushEnabled(currentToken) {
        $("#" + tokenDivId).show();
        $("#" + permissionDivId).hide();
      }

      function updateUIForPushPermissionRequired() {
        $("#" + tokenDivId).hide();
        $("#" + permissionDivId).show();
      }

    }
  };
})(jQuery);
